import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.util.Random;
import java.applet.*;

class PaintovalPane extends JPanel
{
	/*K-Means*/
	int K = 5;
	int N = 50;
	int D = 2;

	Random rand = new Random();

	class Point
	{			
		Point()
		{
			initial();
		}
		
		void initial()
		{
			for (int i = 0; i < D; ++i)
				x[i] = rand.nextDouble()*600;
			clusterNum = -1;
		}
		double x[] = new double[D];
		int clusterNum;	
	};

	Point p[];
	Point centroid[];
	Point oldCentroid[];
	Color colors[];
	/*欧式距离*/
	double Euclidean(Point p1, Point p2)
	{
		double dis = 0;
		
		for (int i = 0; i < D; ++i)
			dis += (p1.x[i]-p2.x[i])*(p1.x[i]-p2.x[i]);
		return Math.sqrt(dis);
	}

	void updateCentroid(int clusterNum)
	{	
		for (int i = 0; i < D; ++i)
			centroid[clusterNum].x[i] = 0;
			
		int clusterSize = 0;
		
		for (int i = 0; i < N; ++i)
			if (p[i].clusterNum == clusterNum)
			{
				clusterSize++;
				
				for (int j = 0; j < D; ++j)
					centroid[clusterNum].x[j] += p[i].x[j];
			}

		if (clusterSize == 0)
			return;
			
		for (int i = 0; i < D; ++i)
			centroid[clusterNum].x[i] /= (double)clusterSize;
	}

	void updateCentroids()
	{
		for (int i = 0; i < K; ++i)
			updateCentroid(i);
	}
	
	void assignPoint(int x)
	{
		double minDis = 99999999;
		int minIndex = 1;
		for (int i = 0; i < K; ++i)
		{
			double curDis = Euclidean(p[x], centroid[i]);
			if (curDis < minDis)
			{
				minDis = curDis;
				minIndex = i;
			}
		}
		p[x].clusterNum = minIndex;
	}

	void assign()
	{
		for (int i = 0; i < N; ++i)
			assignPoint(i);
	}
	
	Boolean samePoint(Point p1, Point p2)
	{
		if (p1.clusterNum != p2.clusterNum)
			return false;
		for (int i = 0; i < D; ++i)
			if (p1.x[i] != p2.x[i])
				return false;
		return true;
	}

	
	Boolean stop()
	{
		for (int i = 0; i < K; ++i)
			if (!samePoint(oldCentroid[i], centroid[i]))	
			{
				for (int j = 0; j < K; ++j)
					copy(oldCentroid[j],centroid[j]);
				return false;
			}
		return true;
	}

	void copy(Point p1, Point p2)
	{
		p1.clusterNum = p2.clusterNum;
		for (int i = 0; i < D; ++i)
			p1.x[i] = p2.x[i];
	}
	void init()
	{
		p = new Point[N];	
		centroid = new Point[K];
		oldCentroid = new Point[K];
		colors = new Color[K];
		for (int i = 0; i < N; ++i)
		{
			p[i] = new Point();
			p[i].initial();
		}

		for (int i = 0; i < K; ++i)
		{
			centroid[i] = new Point();
			oldCentroid[i] = new Point();
			centroid[i].initial();
			oldCentroid[i].initial();
			copy(oldCentroid[i],centroid[i]);
			colors[i] = new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255));
		}
	}

	/*K-Means End*/
	PaintovalPane()
	{
		init();
	}

	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		setBackground(Color.white);
		for (int i = 0; i < N; ++i)
		{
			int x = (int)p[i].x[0], y = (int)p[i].x[1];

			if (p[i].clusterNum == -1)
				g.setColor(Color.black);
			else
				g.setColor(colors[p[i].clusterNum]);

			g.fillOval(x, y, 15, 15);
		}
		for (int i = 0; i < K; ++i) 
		{
			int x = (int)centroid[i].x[0], y = (int)centroid[i].x[1];

			g.setColor(colors[i]);

			g.fillRect(x, y, 15, 15);
		}
	}
}
class Drawing extends JFrame
{
	JButton jButton1 = new JButton("Start");
	JButton jButton2 = new JButton("Step");
	JButton jButton3 = new JButton("Run");
	JLabel label1 = new JLabel("Points");
	JLabel label2 = new JLabel("Clusters");
	JTextField textField1 = new JTextField("This is buffer for text", 15);
	JTextField textField2 = new JTextField("This is buffer for text", 15);
	JPanel jPanel = new JPanel();
	PaintovalPane paint = new PaintovalPane();
	Drawing()
	{
		setTitle("K-Means");
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize (660,710);
		textField1.setText(String.valueOf(paint.N));
		textField2.setText(String.valueOf(paint.K));
		jButton1.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae) 
			{
				int input1 = Integer.parseInt(textField1.getText());
				int input2 = Integer.parseInt(textField2.getText());

				if (input1 > 500 || input1 <= 0)
				{
					JOptionPane.showMessageDialog(null, "Please input the number between 1-500");
				}
				else if (input2 > input1 || input2 <= 0)
				{
					JOptionPane.showMessageDialog(null, "Please input the number between 1-Points");
				}
				else
				{
					paint.N = input1;
					paint.K = input2;
					paint.init();
					paint.repaint();
					jButton2.setText("Step");
					jButton2.setEnabled(true);
					jButton3.setText("Run");
					jButton3.setEnabled(true);
				}
			}
		});
		jButton2.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae) 
			{
				paint.assign();
		
				paint.updateCentroids();

				if (paint.stop())
				{
					jButton2.setText("End");
					jButton2.setEnabled(false);
					jButton3.setText("End");
					jButton3.setEnabled(false);
				}

				paint.repaint();
			}
		});
		jButton3.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae) 
			{
				do
				{
					paint.assign();
		
					paint.updateCentroids();

					paint.repaint();
				}
				while(!paint.stop());

				jButton2.setText("End");
				jButton2.setEnabled(false);
				jButton3.setText("End");
				jButton3.setEnabled(false);
			}
		});
		jPanel.add(label1);
		jPanel.add(textField1);
		jPanel.add(label2);
		jPanel.add(textField2);
		jPanel.add(jButton1);
		jPanel.add(jButton2);
		jPanel.add(jButton3);
		jPanel.setBackground(new Color(1,255,1));
		//jPanel.setSize(660, 100);
		add(BorderLayout.NORTH,jPanel);
		add(BorderLayout.CENTER, paint);
	}
}

public class Hello extends Applet
{
	public static void main(String args[])
	{
		Drawing d = new Drawing();
	}
}